/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn;

/**
 * Base class for all connections between competence modules
 */
abstract class CompetenceConnection extends Connection
{
	/**
	 * Constructor creating a connection by specifying the network parameters
	 * used to calculate the activation passed by this connection and the source
	 * and destination propositions of the connection.
	 * @param params the network parameters controlling runtime dynamics
	 * @param source the source proposition
	 * @param destination the destination proposition
	 */
	CompetenceConnection(NetworkParams params, Proposition source, Proposition destination)
	{
		super(params, source, destination);
	}

	/**
	 * Calculates the activation received directly or indirectly by goal
	 * goalIndex.
	 * @param goalIndex index of the goal from which the activation is originated
	 * @return the activation received directly or indirectly by goal goalIndex.
	 */
	public abstract double getGoalActivation(int goalIndex);
}
