/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

/**
 * Represents a list of propositions logically connected by AND
 */
public class Condition
{
	/** List of preconditions comprising the condition */
	protected final List<Proposition> propositions;

	/**
	 * Default constructor
	 */
	public Condition()
	{
		propositions = new ArrayList<Proposition>();
	}

	/**
	 * Constructor
	 *
	 * @param proposition Proposition to add to this condition
	 */
	public Condition(Proposition proposition)
	{
		this();
		addProposition(proposition);
	}

	/**
	 * Retrieve an iterator to the propositions of this condition
	 *
	 * @return Iterator
	 */
	public Iterator<Proposition> getPropositions()
	{
		return propositions.iterator();
	}

	/**
	 * Calculate the total truth value of the condition from the truth values of
	 * all propositions connected by fuzzy and (product).
	 *
	 * @return Calculated truth value
	 */
	public double getTruthValue()
	{
		Proposition proposition;
		double result = 1.0;

		for (Proposition proposition1 : propositions) {
			proposition = proposition1;
			result *= proposition.getTruthValue();
		}

		return result;
	}

	/**
	 * Adds a proposition to this condition
	 * @param proposition the proposition to add
	 */
	public void addProposition(Proposition proposition)
	{
		propositions.add(proposition);
	}

	/**
	 * Returns the information about this goal.
	 *
	 * @param tabs - the number of tabs used to make indentation in the output.
	 * @return the information about this goal.
	 */
	public String toString(int tabs)
	{
		StringBuffer info = new StringBuffer(100);

		for (Proposition proposition : propositions) {
			if (info.length() > 0) {
				info.append(", ");
			}

			info.append(proposition.toString());
		}

		return info.toString();
	}
}
