/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn;

/**
 * Represents all links that inhibit a competence module by another competence
 * module
 */
public class ConflictorLink extends CompetenceConnection
{
	/**
	 * Calls parent constructor.
	 * @param params the network parameters that define the runtime dynamics of
	 *        the network
	 * @param source the source proposition of the inhibition (negative
	 *        activation)
	 * @param destination the destination proposition
	 */
	public ConflictorLink(NetworkParams params, Proposition source, Proposition destination)
	{
		super(params, source, destination);
	}

	/**
	 * @return the activation of this link
	 */
	@Override
	public double getActivation()
	{
		return activation;
	}

	/**
	 * Calculates the activation received directly or indirectly by goal
	 * goalIndex.
	 * @param goalIndex index of the goal from which the activation is originated
	 * @return the activation received directly or indirectly by goal goalIndex.
	 */
	@Override
	public double getGoalActivation(int goalIndex)
	{
		double proposition;
		double goalActivation;

		goalActivation = sourceProposition.getTransferedActivation(goalIndex);
		proposition = sourceProposition.getTruthValue();

		goalActivation = -networkParams.getDelta() * proposition * goalActivation *
						 ((Effect) destinationProposition).getInfluence();

		if (goalIndex == 0) {
			this.activation = goalActivation;
		} else {
			this.activation += goalActivation;
		}

		return goalActivation;
	}
}
