/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn;

/**
 * Base class for goal tracking, a mechanism to avoid activation spreading
 */
public interface IBaseGoalTrack {
	/**
	 * @return the sum of activation received directly or indirectly by goals
	 */
	double getSumActivation();

	/**
	 * If the passed activation of goal is higher than any previous activation,
	 * take over the new value
	 * @param i index of the goal for which the activation is set
	 * @param val new value for the activation
	 */
	void setActivation(int i, double val);

	/**
	 * @param i index of the goal of which the activation is returned
	 * @return the activation received directly or indirectly by goal i
	 */
	double getActivation(int i);

	/**
	 * @return the number of goals in the network
	 */
	int getNoOfGoals();

	/**
	 * Updates the activations to the newly calculated activations
	 */
	void setToNewActivation();
}
