/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn;

/**
 * Dummy class that is used in case that goaltracking is switched off
 */
class NoGoalTrack implements IBaseGoalTrack
{
	/** activation received directly or indirectly by goal */
	private double activation;

	/** newly calculated activation */
	private double newActivation;

	/**
	 * Default constructor
	 */
	public NoGoalTrack()
	{
		initialize();
	}

	/**
	 * Initializes activation and newActivation fields with zero values
	 */
	void initialize()
	{
		activation = 0.0;
		newActivation = 0.0;
	}

	/**
	 * @return the number of goals in the network
	 */
	@Override
	public int getNoOfGoals()
	{
		return 1;
	}

	/**
	 * @return the activation received directly or indirectly by goals
	 */
	@Override
	public double getSumActivation()
	{
		return activation;
	}

	/**
	 * If the passed activation of goal is higher than any previous activation,
	 * take over the new value
	 *
	 * @param i index of the goal for which the activation is set
	 * @param val new value for the activation
	 */
	@Override
	public void setActivation(int i, double val)
	{
		newActivation += val;
	}

	/**
	 * @param i index of the goal of which the activation is returned
	 * @return the activation received directly or indirectly by goal
	 */
	@Override
	public double getActivation(int i)
	{
		return activation;
	}

	/**
	 * Updates the activation to the newly calculated activation
	 */
	@Override
	public void setToNewActivation()
	{
		activation = newActivation;
		newActivation = 0.0;
	}
}
