/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebn.observation;

import java.util.ArrayList;
import java.util.List;

/**
 * Class that provides observation functionality for th ebn
 * @author Thomas Rinklin
 *
 */
public class EbnSubject implements ISubscribeUnsubscribe
{
	/** the list of observers that are notified */
	private final List<IEbnObserver> observers;

	/**
	 * Default constructor creating the observer list
	 */
	public EbnSubject()
	{
		super();
		this.observers = new ArrayList<IEbnObserver>();
	}

	@Override
	public void attach(IEbnObserver observer)
	{
		if (!observers.contains(observer)) {
			observers.add(observer);
		}
	}

	@Override
	public boolean detach(IEbnObserver observer)
	{
		return observers.remove(observer);
	}

	/**
	 * Removes all observers from the list of observers
	 */
	public void detachAll()
	{
		observers.clear();
	}

	/**
	 * informs all observers that values has changed
	 */
	public void onValueChange()
	{
		for (IEbnObserver observer : observers) {
			observer.valuesChanged();
		}
	}

	/**
	 * informs all observers that structure has changed
	 */
	public void onStructureChange()
	{
		for (IEbnObserver observer : observers) {
			observer.structureChanged();
		}
	}
}
