/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.ebnAccess;

import java.io.File;

import kdo.ebnDevKit.agent.IEbnAgent;
import kdo.ebnDevKit.agent.IEbnAgentCreator;
import kdo.ebnDevKit.ebnAccess.impl.EbnAccess;

/**
 * Factory class for EbnAccess objects
 * @author Thomas Rinklin
 *
 */
public class EbnAccessFactory
{
	/** reference to the agent creator */
	private IEbnAgentCreator agentCreator;

	/**
	 * Adds a agent creator to the controller
	 * @param agentCreator the agent creator
	 */
	public void setAgentCreator(IEbnAgentCreator agentCreator)
	{
		this.agentCreator = agentCreator;
	}

	/**
	 * creates an ebnaccess object with an empty ebn
	 * @return the created ebnAccess object
	 */
	public IEbnAccess creatEmptyEbnAccess()
	{
		IEbnAgent agent = agentCreator.createAgent();
		return new EbnAccess(agent);
	}

	/**
	 * creates an ebnaccess object with an empty ebn from file
	 * @param f the ebn file
	 * @return the created ebnAccess object
	 */
	public IEbnAccess createEbnFromFile(File f)
	{
		IEbnAgent agent = agentCreator.createAgent();
		return new EbnAccess(agent, f);
	}
}
