/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.ebnAccess;

import kdo.ebn.IEBNPerception;
import kdo.ebnDevKit.ebnAccess.impl.elements.EbnEffect;

/**
 * interface to a perception
 * @author Thomas Rinklin
 */
public interface IEbnPerception {
	/**
	 * returns the name of the perception
	 * @return the name of the perception
	 */
	String getName();

	/**
	 * returns the activation of the perception
	 * @return the activation of the perception
	 */
	double getActivation();

	/**
	 * returns the truth value of the perception
	 * @return the truth value of the perception
	 */
	double getTruthValue();

	/**
	 * creates proposition upon the perception
	 * @param bVal true if the proposition is negated, false if not
	 * @return created proposition
	 */
	IEbnProposition createProposition(boolean bVal);

	/**
	 * factory method to create an effect with this proposition
	 * @param negated if the proposition is negated
	 * @param probability the probability
	 * @return the proposition
	 */
	EbnEffect createEffect(boolean negated, double probability);

	/**
	 * interface to a proposition
	 * @author Thomas Rinklin
	 */
	public interface IEbnProposition {
		/**
		 * returns the underlying perception
		 * @return the underlying perception
		 */
		IEbnPerception getPerception();

		/**
		 * @return true if the proposition is negated, false if not
		 */
		boolean isNegated();
	}

	/**
	 * interface to an effect
	 * @author Thomas Rinklin
	 */
	public interface IEbnEffect extends IEbnProposition {
		/**
		 * returns the probability
		 * @return the probability
		 */
		double getProbability();
	}

	/**
	 * returns if the passed belief is the underlying belief
	 * @param iBelief belief to check
	 * @return true if the passed belief is the underlying belief, false if not
	 */
	boolean isBelief(IEBNPerception iBelief);
}