/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.ebnAccess;

import kdo.ebn.IResourceBelief;

/**
 * interface to a resource
 * @author Thomas Rinklin
 */
public interface IEbnResource {
	/**
	 * checks if this resource wraps the passed resource belief
	 * @param iResourceBelief resource belief to check
	 * @return true if the resource belief matches to this resource, false if not
	 */
	boolean isResource(IResourceBelief iResourceBelief);

	/**
	 * returns the name of the resource
	 * @return the name of the resource
	 */
	String getName();

	/**
	 * factory method to create a resource proposition
	 * @param amountUsed the the resource amount used by the proposition
	 * @return the created resource proposition
	 */
	IEbnResourceProposition creatProposition(int amountUsed);

	public int getAmountAvailable();

	public int getMaximalAmount();

	/**
	 * interface to a resource proposition
	 * @author Thomas Rinklin
	 */
	public interface IEbnResourceProposition {
		/**
		 * checks if this resource proposition matches to a resource
		 * @param resource resource to check
		 * @return true if the resource matches to this resource proposition
		 */
		boolean isResource(IEbnResource resource);

		/**
		 * returns the underlying resource
		 * @return the underlying resource
		 */
		IEbnResource getResource();

		/**
		 * returns the amount of resource which this proposition uses
		 * @return the amount of resource which this proposition uses
		 */
		int getAmountUsed();
	}
}