/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.ebnAccess.impl.elements;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import kdo.ebn.Goal;
import kdo.ebnDevKit.ebnAccess.IEbnGoal;
import kdo.ebnDevKit.ebnAccess.IEbnPerception;
import kdo.ebnDevKit.ebnAccess.IEbnPerception.IEbnProposition;

/**
 * represents a goal
 * @author Thomas Rinklin
 */
public class EbnGoal implements IEbnGoal
{
	/** link to the goal */
	private final Goal goal;

	/** goal condition */
	private final IEbnProposition goalCondition;

	/** relevance propositions */
	private final List<EbnProposition> ebnRelevanceConditions;

	/**
	 * create a ebnGoal object
	 * @param goal the existing goal
	 * @param ebnProposition underlaying proposition
	 */
	public EbnGoal(Goal goal, IEbnProposition ebnProposition)
	{
		this.goalCondition = ebnProposition;
		this.goal = goal;
		// this.ebnAccess = ebnAccess;
		this.ebnRelevanceConditions = new ArrayList<EbnProposition>();
	}

	@Override
	public String getName()
	{
		return goal.getName();
	}

	@Override
	public int getIndex()
	{
		return goal.getIndex();
	}

	@Override
	public double getActivation()
	{
		return goal.getActivation();
	}

	@Override
	public double getImportance()
	{
		return goal.getImportance();
	}

	@Override
	public double getRelevance()
	{
		return goal.getRelevance();
	}

	@Override
	public double getGoalConditionTruthValue()
	{
		return goal.getGoalCondition().getTruthValue();
	}

	@Override
	public IEbnProposition getGoalCondition()
	{
		return goalCondition;
	}

	@Override
	public Iterator<? extends IEbnProposition> getRelevanceConditions()
	{
		return ebnRelevanceConditions.iterator();
	}

	/**
	 * returns goal link
	 * @return goal link
	 */
	public Goal getGoal()
	{
		return goal;
	}

	/**
	 * adds proposition
	 * @param ebnProp proposition to add
	 */
	public void addProposition(EbnProposition ebnProp)
	{
		ebnRelevanceConditions.add(ebnProp);
	}

	@Override
	public boolean isPerceptionUsed(IEbnPerception perception)
	{
		if (goalCondition.getPerception() == perception)
			return true;

		for (IEbnProposition prop : ebnRelevanceConditions) {
			if (prop.getPerception() == perception)
				return true;
		}
		return false;
	}
}
