/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.gui;

import java.awt.BorderLayout;
import java.awt.event.ContainerEvent;
import java.awt.event.ContainerListener;

import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import kdo.ebnDevKit.ebnAccess.EbnAccessFactory;
import kdo.ebnDevKit.gui.ebnGraphPanel.GraphConfiguration;

/**
 * EbnDevKit main panel
 * @author Thomas Rinklin
 */
public class EbnDevKit extends JPanel
{
	private static final long serialVersionUID = -6756767454758078432L;

	/** menu bar */
	private final MenuBar menuBar;

	/** tabbed pane for the ebn views */
	private final EbnTabbedPane tabbedPane;

	/** enclosing frame */
	private JFrame frame;

	public EbnDevKit(EbnAccessFactory ebnAccessFactory)
	{
		GraphConfiguration graphConfiguration = new GraphConfiguration();
		ApplicationActionListeners actionListeners =
				new ApplicationActionListeners(this, ebnAccessFactory, graphConfiguration);
		tabbedPane = new EbnTabbedPane();
		menuBar = new MenuBar(actionListeners, tabbedPane);
		actionListeners.setMenuBar(menuBar);
		actionListeners.setTabbedPane(tabbedPane);

		setLayout(new BorderLayout());
		add(menuBar, BorderLayout.NORTH);
		add(tabbedPane, BorderLayout.CENTER);

		addTabbedPaneChangeListener();

		menuBar.updateButtons();
	}

	private void addTabbedPaneChangeListener()
	{
		tabbedPane.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e)
			{
				menuBar.updateButtons();
			}
		});

		tabbedPane.addContainerListener(new ContainerListener() {
			@Override
			public void componentRemoved(ContainerEvent arg0)
			{
				menuBar.updateButtons();
			}

			@Override
			public void componentAdded(ContainerEvent arg0)
			{
				menuBar.updateButtons();
			}
		});
	}

	/**
	 * sets the enclosing frame
	 * @param frame enclosing frame
	 */
	public void setEnclosingFrame(JFrame frame)
	{
		this.frame = frame;
	}

	/**
	 * returns the enclosing frame
	 * @return enclosing frame
	 */
	public JFrame getFrame()
	{
		return this.frame;
	}
}
