/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.ebnDevKit.gui.ebnGraphPanel.guiElements.links;

import kdo.ebnDevKit.gui.ebnGraphPanel.GraphConfiguration;
import kdo.ebnDevKit.gui.ebnGraphPanel.guiElements.GuiElement;
import kdo.ebnDevKit.gui.ebnGraphPanel.guiElements.nodes.GuiNode;

/**
 * base class for all connections between ebn gui elements
 * @author Thomas Rinklin
 *
 */
public abstract class GuiConnection<S extends GuiNode, T extends GuiNode> extends GuiElement
{
	protected final S source;

	protected final T target;

	public GuiConnection(S source, T target)
	{
		this.source = source;
		this.target = target;
	}

	protected boolean shouldPrint(GraphConfiguration.ConnectionVisibility conVis)
	{
		switch (conVis) {
		case ALWAYS:
			return true;

		case TARGET_OR_SOURCE_SELECTED:
			return targetSelected() || sourceSelected();

		case TARGET_SELECTED:
			return targetSelected();

		case SOURCE_SELECTED:
			return sourceSelected();

		case NEVER:
			return false;
		}
		return false;
	}

	private boolean targetSelected()
	{
		return target.isSelected();
	}

	private boolean sourceSelected()
	{
		return source.isSelected();
	}
}
