/*
 * Copyright (c) 2008 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.neuralNetwork.feedforward;

import kdo.neuralNetwork.transfer.ITransferFunction;
import kdo.util.IRandomSource;

/**
 * Represents a layer of nodes and their incoming weights in a neural network
 */
public class BPGHiddenLayer extends Layer
{
	/** */
	private static final long serialVersionUID = 1L;

	/**
	 *
	 * @param noOfInputs the number of input neurons (including bias)
	 * @param noOfOutputs the number of output neurons
	 * @param transfer the transfer function to use
	 * @param rand the source for random numbers
	 */
	public BPGHiddenLayer(int noOfInputs, int noOfOutputs, ITransferFunction transfer, IRandomSource rand)
	{
		super(noOfInputs, noOfOutputs, transfer, rand);
	}

	/**
	 * @param deltaNextLayer
	 * @return
	 */
	@Override
	public float[] calculateDelta(float[] deltaNextLayer)
	{
		float[] result = new float[outputLayer.length];
		for (int i = 0; i < outputLayer.length; i++) {
			result[i] = transferFunction.transferDerivative(outputLayer[i]) * deltaNextLayer[i];
		}
		// System.out.println("OutputDelta: " + Arrays.toString(result));
		return result;
	}
}
