/*
 * Copyright (c) 2008 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.neuralNetwork.feedforward;

import kdo.neuralNetwork.transfer.ITransferFunction;
import kdo.util.IRandomSource;

/**
 * Represents a layer of nodes and their incoming weights in a neural network
 */
public class BPGOutputLayer extends Layer
{
	/**	 */
	private static final long serialVersionUID = 1L;

	/**
	 *
	 * @param noOfInputs the number of input neurons (including bias)
	 * @param noOfOutputs the number of output neurons
	 * @param transfer the transfer function to use
	 * @param rand the source for random numbers
	 */
	public BPGOutputLayer(int noOfInputs, int noOfOutputs, ITransferFunction transfer, IRandomSource rand)
	{
		super(noOfInputs, noOfOutputs, transfer, rand);
	}

	/**
	 * Calculates the delta value for an output layer as g'(ini) * (Ti - Oi)
	 * @param input
	 * @param goalOutput
	 * @return
	 */
	@Override
	public float[] calculateDelta(float[] goalOutput)
	{
		float[] result = new float[outputLayer.length];
		for (int i = 0; i < weight[0].length; i++) {
			float out = outputLayer[i];
			result[i] = transferFunction.transferDerivative(out) * (goalOutput[i] - out);
		}

		// System.out.println("OutputDelta: " + Arrays.toString(result));
		return result;
	}
}
