/*
 * Copyright (c) 2012 Klaus Dorer
 * Hochschule Offenburg
 */
package kdo.neuralNetwork.output;

/**
 * Only the output with highest value is returned
 * @author kdorer
 */
public class WinnerTakesAll implements IOutputFunction
{
	/** true if the winning output should return a value specified from outside */
	private boolean useExternalValue;

	/** the value to use for winning output if useExternalValue is true */
	private float outsideWinnerValue;

	/**
	 * @param useExternalValue true if the winning output should return a value
	 *        specified from outside
	 * @param outsideWinnerValue the value to use for winning output if
	 *        useExternalValue is true
	 */
	public WinnerTakesAll(boolean useExternalValue, float outsideWinnerValue)
	{
		this.useExternalValue = useExternalValue;
		this.outsideWinnerValue = outsideWinnerValue;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see kdo.neuralNetwork.output.IOutputFunction#output(float[])
	 */
	@Override
	public float[] output(float[] output)
	{
		float candidate = output[0];
		int candidateIndex = 0;
		for (int i = 1; i < output.length; i++) {
			float next = output[i];
			if (next > candidate) {
				candidate = next;
				candidateIndex = i;
			}
		}

		for (int i = 0; i < output.length; i++) {
			if (i == candidateIndex) {
				if (useExternalValue) {
					output[i] = outsideWinnerValue;
				}
			} else {
				output[i] = 0;
			}
		}
		return output;
	}
}
