/* Copyright 2009 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Rajit Shahi
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 */

package kdo.search.runs.averageOutStrategies;

import kdo.domain.IUtilityCalculator;
import kdo.domain.UtilityCalculatorParameters;

/**
 * Does a serial run of all trials, expecting that the server is already
 * running.
 * @author kdorer
 */
public class SerialAverageOutStrategy extends AverageOutStrategyBase
{
	public SerialAverageOutStrategy(int averageOutRuns, boolean verbose, IDomainVisitor visitor)
	{
		super(averageOutRuns, verbose, visitor);
	}

	@Override
	public double getAverageUtility(AverageOutState state)
	{
		UtilityCalculatorParameters ucParameters = createUCParameters(0);
		IUtilityCalculator utilityCalculator = null;
		int repeatCount = 0;
		while (state.count < averageOutRuns) {
			utilityCalculator = state.getUtilityCalculator(ucParameters);
			double util = utilityCalculator.singleRunUtility(state.count, averageOutRuns, repeatCount);
			if (util == Double.NEGATIVE_INFINITY) {
				// we do not count invalid measurement runs
				repeatCount++;
				continue;
			}
			repeatCount = 0;

			addValues(utilityCalculator.getProperties(), util, state);
		}

		if (utilityCalculator != null) {
			utilityCalculator.printParams();
		}
		finishedCalculation(state);
		return state.averageUtility;
	}
}
