/* Copyright 2009 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Rajit Shahi
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 */

package kdo.search.runs.averageOutStrategies;

import java.util.HashMap;
import java.util.Map;

/**
 * Average out strategy that expects to be called from multiple threads at the
 * same time while running.
 * @author kdorer
 */
public class ThreadedAverageOutStrategy extends AverageOutStrategyBase
{
	private Map<String, SerialAverageOutStrategy> callers;

	public ThreadedAverageOutStrategy(int averageOutRuns, boolean verbose, IDomainVisitor visitor)
	{
		super(averageOutRuns, verbose, visitor);
		callers = new HashMap<>();
	}

	@Override
	public double getAverageUtility(AverageOutState state)
	{
		SerialAverageOutStrategy serial = new SerialAverageOutStrategy(averageOutRuns, verbose, visitor);
		callers.put(Thread.currentThread().getName(), serial);
		return serial.getAverageUtility(state);
	}
}
