/*
 * Copyright (c) 2008 Klaus Dorer Hochschule Offenburg
 */
package kdo.sort;

import java.util.Arrays;
import java.util.Random;

public class SortMain
{
	static int NUMBERS[] = {1, 3, 2, 5, 7, 6, 4}; // "Standard"-Array

	public static void main(String args[])
	{
		// use this call to measure performance
		if (true) {
			sortOnce(args);
			testSort(args);
		}

		// int i;
		// for (i = 0; i < 1000; i++) {
		// testSort(args);
		// }
		// System.out.println(i + " tests finished");
	}

	/**
	 * @param args
	 */
	private static void sortOnce(String[] args)
	{
		Integer t[] = getRandomArray(args, 251);

		doSort(t, new BubbleSort());
		doSort(t, new InsertionSort());
		doSort(t, new ShellSort());
		doSort(t, new SelectionSort());
		doSort(t, new QuickSort(null));
		doSort(t, new MergeSort());
		doSort(t, new HeapSort());
	}

	/**
	 * @param t
	 * @param sorter
	 */
	private static void doSort(Integer[] t, ISortAlgorithm sorter)
	{
		Integer copy[] = t.clone();
		long start = System.currentTimeMillis();
		sorter.sort(copy);
		long end = System.currentTimeMillis();

		System.out.println(sorter.getName() + ": elements: " + t.length + " Runtime: " + (end - start));
	}

	/**
	 * @param args
	 */
	private static void testSort(String[] args)
	{
		Integer t[] = getRandomArray(args, System.currentTimeMillis());
		Integer copy1[] = t.clone();
		Integer copy2[] = t.clone();

		SortAlgorithmBase sorter = new QuickSort(null);
		// SortAlgorithmBase sorter = new InsertionSort();
		// sorter.printArray(t);
		sorter.sort(copy1);
		Arrays.sort(copy2);
		if (!Arrays.deepEquals(copy1, copy2)) {
			System.out.println("Error: " + Arrays.toString(t) + "\nWas sorted to: " + Arrays.toString(copy1));
		}
		sorter.printArray(copy1);
	}

	/**
	 * If a number is passed in command line args, that number of elements is
	 * generated in random order, if not a standard array is used
	 *
	 * @param args the command line arguments
	 * @return an array of Integer elements to be sorted
	 */
	private static Integer[] getRandomArray(String[] args, long randomSeed)
	{
		Integer[] result;
		int vec[] = NUMBERS;
		if (args.length != 0) {
			// generate random number of Integers to sort
			int size = Integer.valueOf(args[0]).intValue();
			result = new Integer[size];
			Random rand = new Random(randomSeed);
			for (int i = 0; i < size; i++) {
				result[i] = new Integer(Math.abs(rand.nextInt()) % (size * 2));
			}
		} else {
			// use default array to sort
			result = new Integer[vec.length];
			for (int i = 0; i < vec.length; i++) {
				result[i] = new Integer(vec[i]);
			}
		}
		return result;
	}
}
