/*******************************************************************************
 * Copyright 2008, 2012 Hochschule Offenburg
 * Klaus Dorer, Mathias Ehret, Stefan Glaser, Thomas Huber, Fabian Korak,
 * Simon Raffeiner, Srinivasa Ragavan, Thomas Rinklin,
 * Joachim Schilling, Ingo Schindler, Rajit Shahi, Bjoern Weiler
 *
 * This file is part of magmaOffenburg.
 *
 * magmaOffenburg is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * magmaOffenburg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with magmaOffenburg. If not, see <http://www.gnu.org/licenses/>.
 *******************************************************************************/
package kdo.util.parameter;

import java.io.Serializable;
import java.util.Collections;
import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Base class for sets of parameters used to modify Behaviors
 * @author Klaus Dorer
 */
@SuppressWarnings("serial")
public class ParameterList implements Serializable, IParameterList
{
	/** parameters used by the underlying behavior */
	protected Map<String, Parameter> parameters;

	public ParameterList()
	{
		parameters = new LinkedHashMap<>();
	}

	@Override
	public Map<String, Parameter> getParameters()
	{
		return Collections.unmodifiableMap(parameters);
	}

	final public float get(String key)
	{
		if (!parameters.containsKey(key)) {
			return 0;
		}
		return parameters.get(key).getValue();
	}

	final public void put(String key, float value)
	{
		if (!parameters.containsKey(key)) {
			parameters.put(key, new Parameter(key));
		}
		parameters.get(key).setValue(value);
	}

	@Override
	public String toString()
	{
		return parameters.toString();
	}

	@Override
	public String getParamsString()
	{
		StringBuilder builder = new StringBuilder(100);
		for (String key : parameters.keySet()) {
			builder.append("\nput(").append(getParamCode(key));
			builder.append(", ").append(get(key)).append("f);");
		}
		return builder.toString();
	}

	protected String getParamCode(String key)
	{
		return "Param." + key;
	}
}
